import { SocketEvent } from '@/socket/socket-event';
import { AlertService, Modal, ModalService } from '@zeedhi/common';
import { Config, Http, IDictionary, Metadata } from '@zeedhi/core';
import { TekGrid, TekGridColumn, ITekGridColumn, IFilterPropsComponent, TekMemoryDatasource } from '@zeedhi/teknisa-components-common';
import { io, Socket } from 'socket.io-client';

export class MonitoringController {
  public socket?: Socket;

  public isVisibleModalColumn: boolean = false;

  private monitoringModal!: Modal;

  private cdfilial: any;

  private clickedAssConfig: any;

  private assConfig: any;

  private satColumnId: any;

  private columnsId: any = [];

  private socketArgs = {
    email: 'assistant@teknisa.com',
    password: 'nineninenine',
    slave: false,
  };

  public gridBeforeApplyFilter() {
    const { datasource } = Metadata.getInstance<TekGrid>('monitoring-grid');
    const filter = (datasource as TekMemoryDatasource).dynamicFilter;
    localStorage.setItem('LAST_FILTER_MONITORING', JSON.stringify(filter));
  }

  public async gridMounted() {
    const grid = Metadata.getInstance<TekGrid>('monitoring-grid');

    try {
      grid.datasource.loading = true;
      this.monitoringModal = await ModalService.createFromJson('monitoringModal', true);

      if (!this.socket) {
        this.socket = io(Config.endPoint, {
          transports: ['websocket', 'polling'],
          query: {
            page: 'monitoring',
          },
        });
      }

      const reqParams = '/data/parametrizacoes';
      const columns = await Http.get(reqParams);
      this.addColumns(columns, grid);

      const reqMonitoring = '/data/monitoring';
      const monitoring = await Http.get(reqMonitoring);

      this.setLastFilter(columns, grid);
      grid.datasource.data = monitoring.data.data.sort(this.orderGridData.bind(this));
      grid.datasource.loading = false;
      this.socketListener(columns);
      this.socketEmit();
    } catch (error) {
      grid.datasource.loading = false;
      this.showAlert(error, 'error');
    }
  }

  private setLastFilter(columns: any, grid: TekGrid) {
    const lastFilter = localStorage.getItem('LAST_FILTER_MONITORING');
    if (lastFilter) {
      const filter = JSON.parse(lastFilter);
      if (Object.keys(filter).length > 0) {
        let actualColumns = columns.data.data.filter((element: any) => element.ATIVO === 'S' && element.TIPO !== 'C')
                                                .map((element: any) => element.ID);
        // Colunas fixas do grid não podem ser removidas
        actualColumns = actualColumns.concat(['id', 'name', 'situacao', 'sat']);
        Object.keys(filter).forEach((item: any) => {
          if (!actualColumns.includes(item)) {
            delete filter[item];
          }
        });
        (grid.datasource as TekMemoryDatasource).dynamicFilter = filter;
      }
    }
  }

  private socketEmit() {
    this.socket?.emit(SocketEvent.CONNECTION);
    this.socket?.emit('login', this.socketArgs);
    this.socket?.emit('getLatestMonitoringResult');
  }

  private socketListener(columns: any) {
    this.socket?.on('monitoringResultSummary', async (response: any) => {
      const UNITS = Object.keys(response);
      const grid = Metadata.getInstance<TekGrid>('monitoring-grid');
      let updatedUnits = 0;
      grid.datasource.data.every((unit: any) => {
        if (UNITS.includes(unit.CDFILIAL)) {
          Object.assign(unit, response[unit.CDFILIAL]);
          updatedUnits += 1;
          if (UNITS.length === updatedUnits) return false;
        }
        return true;
      });
      this.updateGridData(grid.datasource.data);
    });
    this.socket?.on('login', async (response: any) => {
      const grid = Metadata.getInstance<TekGrid>('monitoring-grid');
      const { UNITS, STATUS }: any = response;
      if (UNITS.length) {
        let updatedUnits = 0;
        grid.datasource.data.every((unit: any) => {
          if (UNITS.includes(unit.CDFILIAL)) {
            unit.websocket = STATUS;
            updatedUnits += 1;
            if (UNITS.length === updatedUnits) return false;
          }
          return true;
        });
        this.updateGridData(grid.datasource.data);
      }
    });

    this.socket?.on('disconnect', () => {
      this.showAlert('A página será recarregada para restabelecer a conexão.', 'error');
      setTimeout(() => {
        document.location.reload();
      }, 5000);
    });
    columns.data.data.forEach((element: any) => {
      if (element.ATIVO === 'S' && element.TIPO !== 'C') {
        this.socket?.on(element.ID, async (response: any) => {
          const grid = Metadata.getInstance<TekGrid>('monitoring-grid');
          const unit: any = grid.datasource.data.find(({ CDFILIAL }: any) => CDFILIAL === response.CDFILIAL);
          unit[response.ID] = response.STATUS;
          if (element.TIPO === 'I' && element.NOME === 'SAT') {
            unit.sat = response.STATUS === 'S';
          }
          this.updateGridData(grid.datasource.data);
        });
      }
    });
  }

  private addColumns(columns: any, grid: TekGrid) {
    this.assConfig = columns.data.data;
    columns.data.data.forEach((element: any) => {
      if (element.ATIVO === 'S' && element.TIPO !== 'C') {
        this.columnsId.push(element.ID);
        if (element.NOME === 'SAT') {
          this.satColumnId = element.ID;
        }
        const newColumn = this.getNewColumn(element, grid);
        if (element.NOME !== 'SAT') {
          newColumn.filterProps = this.getNewColumnFilterProps(element);
        }
        grid.columns.push(newColumn);
      }
    });
  }

  private getNewColumn(element: any, grid: TekGrid) {
    const newColumn: ITekGridColumn = {
      name: element.ID,
      label: element.NOME,
      type: 'action',
      maxWidth: '100px',
      children: [
        {
          name: `button_${element.ID}`,
          component: 'ZdButton',
          small: true,
          icon: true,
          iconName: 'mdi-brightness-1',
          conditions: {
            color: this.getButtonColor.bind(this),
          },
          events: {
            click: this.buttonClickMonitoring.bind(this),
          },
        },
      ],
    };
    return new TekGridColumn(newColumn, grid);
  }

  private getNewColumnFilterProps(element: any) {
    const filterProps: IFilterPropsComponent = {
      name: element.ID,
      label: element.NOME,
      relation: 'AND',
      operation: 'EQUALS',
      grid: {
        sm: 3,
      },
      component: 'ZdSelect',
      autocomplete: false,
      dataText: ['text'],
      dataValue: 'value',
      datasource: {
        data: [
            {
              value: 'S',
              text: 'Sucesso',
            },
            {
              value: 'N',
              text: 'Erro',
            },
        ],
      },
    };
    return filterProps;
  }

  public closeModal() {
    this.monitoringModal.hide();
  }

  public modalGridMounted() {
    const modalGrid = Metadata.getInstance<TekGrid>('monitoring-modal-grid');
    modalGrid.datasource.uniqueKey = 'NAME';
    const column = <TekGridColumn> modalGrid.getColumn('NAME');
    modalGrid.datasource.loading = true;
    this.clickedAssConfig.CDFILIAL = this.cdfilial;

    this.socket?.emit('detailMonitoring', this.clickedAssConfig);
    this.socket?.on('detailMonitoringResult', async (response: any) => {
      this.isVisibleModalColumn = false;
      if (response && response.rows && response.rows.length) {
        if (this.clickedAssConfig.NOME === 'SAT') {
          this.isVisibleModalColumn = true;
        } else if (response.metaData) {
          response.metaData.forEach((element: any, key: any) => {
            const newColumn = new TekGridColumn({ ...column }, modalGrid);
            newColumn.name = key;
            newColumn.label = element.name;
            newColumn.isVisible = true;
            modalGrid.columns.push(newColumn);
          });
        } else {
          Object.keys(response.rows[0]).forEach((element: any) => {
            let newColumn;
            // S => Serviços - Loja
            // E => Apache - Loja
            // I => Interno
            const {
              TIPO,
              NOME,
            } = this.clickedAssConfig;
            let buildStatusButton = ['S', 'E'].includes(TIPO);
            buildStatusButton = buildStatusButton || (TIPO === 'I' && NOME === 'Serviços Updater');
            if (buildStatusButton && element === 'STATUS') {
              newColumn = this.getServicesStatusColumn(element, modalGrid);
              modalGrid.datasource.uniqueKey = 'DESCRICAO';
            } else {
              newColumn = new TekGridColumn({ ...column }, modalGrid);
              newColumn.name = element;
              newColumn.label = element;
              newColumn.isVisible = true;
            }
            modalGrid.columns.push(newColumn);
          });
        }
        modalGrid.datasource.updateData(response.rows);
      }
      modalGrid.datasource.loading = false;
    });
  }

  private getServicesStatusColumn(element: any, grid: TekGrid) {
    const newColumn: ITekGridColumn = {
      name: element,
      label: element,
      type: 'action',
      maxWidth: '100px',
      fixed: true,
      children: [
        {
          name: `button_${element}`,
          component: 'ZdButton',
          small: true,
          icon: true,
          iconName: 'mdi-brightness-1',
          conditions: {
            color: this.getServicesButtonColor.bind(this),
          },
        },
      ],
    };
    return new TekGridColumn(newColumn, grid);
  }

  public gridBeforeDestroy() {
    this.monitoringModal?.destroy();
    this.socket?.removeAllListeners();
    this.socket?.disconnect();
  }

  public getButtonColor({ column, row }: IDictionary<any>) {
    let cor;
    const isActiveConn = row.websocket === 'S';
    if (column.name === 'websocket') {
      cor = isActiveConn ? 'green' : 'red';
    } else if (this.satColumnId && column.name === this.satColumnId && row.sat === 'N') {
      cor = 'gray';
    } else if (column.label === 'Serviços Updater') {
      if (!isActiveConn) return 'red';
      if (!row[column.name]) return 'gray';
      cor = row[column.name] === 'S' ? 'green' : 'red';
    } else {
      cor = row[column.name] === 'S' && isActiveConn ? 'green' : 'red';
    }
    return cor;
  }

  public getSatButtonColor({ row }: IDictionary<any>) {
    return row.sat ? 'green' : 'red';
  }

  public getServicesButtonColor({ row }: IDictionary<any>) {
    return row.STATUS === 'S' ? 'green' : 'red';
  }

  public buttonClickMonitoring(monitor:any) {
    if (monitor.component.color === 'green') {
      this.showAlert('Tudo certo com esse monitoramento!', 'success');
    } else if (monitor.column.name === this.satColumnId && monitor.row.sat === 'N') {
      this.showAlert('Essa filial não utiliza modelo fiscal SAT', '#e68129');
    } else if (monitor.row.websocket === 'S') {
      if (monitor.column.label === 'Serviços Updater' && monitor.component.color === 'gray') {
        return this.showAlert('Essa filial não possui serviços gerenciados pelo Assistant', '#e68129');
      }
      this.cdfilial = monitor.row.CDFILIAL;
      this.clickedAssConfig = this.assConfig.find((element: any) => element.ID === monitor.column.name);
      this.monitoringModal.children[1].title = this.clickedAssConfig.DESCRICAO;
      this.monitoringModal.show();
    } else {
      this.showAlert('A conexão deve estar ativa para visualizar os monitoramentos', '#e68129');
    }
  }

  private showAlert(message: any, colorStr: string) {
    AlertService.show({
      name: 'info-alert',
      text: message,
      color: colorStr,
      right: true,
      top: true,
      showDismiss: true,
      timeout: 10000,
    });
  }

  private updateGridData(data: any) {
    const grid = Metadata.getInstance<TekGrid>('monitoring-grid');
    grid.datasource.updateData(data.sort(this.orderGridData.bind(this)));
  }

  private orderGridData(item1: any, item2: any) {
    if (item1.websocket === 'N' && item2.websocket === 'S') {
      return -1;
    }
    if (item1.websocket === 'S' && item2.websocket === 'N') {
      return 1;
    }

    const some1 = this.columnsId.some((columnId: any) => item1[columnId] === 'N' && item2[columnId] === 'S');
    if (some1) {
      return -1;
    }
    const some2 = this.columnsId.some((columnId: any) => item1[columnId] === 'S' && item2[columnId] === 'N');
    if (some2) {
      return 1;
    }

    if (item1.name < item2.name) {
      return -1;
    }
    if (item1.name > item2.name) {
      return 1;
    }

    return 0;
  }
}
